@@ -118,6 +118,9 @@ def check_version_2(dataset):
     """
     if float(dataset.get('version')) >= 2.0 \
             if dataset.get('version') else False:
+        if 'id' not in dataset:
+            # Some datasets claim to be 2.0, but don't have version 2 structure.
+            return False
         return True
     else:
         return False
@@ -183,7 +186,11 @@ def get_dimensions(js_dict, naming):
     else:
         dimension_dict = js_dict['dimension']
     for dim in dimension_dict['id']:
-        dim_name = js_dict['dimension'][dim]['label']
+        if 'label' in js_dict['dimension'][dim]:
+            dim_name = js_dict['dimension'][dim]['label']
+        else:
+            # All datasets don't have labels for dimensions
+            dim_name = dim
         if not dim_name:
             dim_name = dim
         if naming == 'label':
@@ -434,9 +441,124 @@ def from_json_stat(datasets, naming='label', value='value'):
     return results
 
 
+def _round_decimals(dataframe, units, roles, value):
+    """Round values according to units definition."""
+    if not isinstance(dataframe, pd.DataFrame):
+        return dataframe
+    if not isinstance(units, dict):
+        return dataframe
+    if not isinstance(roles, dict):
+        return dataframe
+    if not isinstance(value, str):
+        return dataframe
+
+    df = dataframe.copy(deep=True)
+    for unit in units.keys():
+        if unit in roles['metric']:
+            if '*' in units[unit].keys():
+                decimals = units[unit]['*'].get('decimals')
+                if decimals is not None:
+                    df[value] = df[value].round(decimals)
+            else:
+                for label in units[unit].keys():
+                    decimals = units[unit][label].get('decimals')
+                    if decimals is not None:
+                        for idx, row in df.iterrows():
+                            if row[unit] == label:
+                                df.at[
+                                    idx, value] = df.at[
+                                        idx, value].round(decimals)
+
+    return df
+
+
+def _add_units_to_categories(categories, units, roles):
+    """Add units to categories according to units definition."""
+    if not isinstance(categories, list):
+        return categories
+    if not isinstance(units, dict):
+        return categories
+    if not isinstance(roles, dict):
+        return categories
+
+    for unit in units.keys():
+        if unit in roles['metric']:
+            for idx, category in enumerate(categories):
+                if unit == list(category)[0]:
+                    if '*' in units[unit].keys():
+                        units_updated = {}
+                        for c in categories[idx][unit]['category']['label']:
+                            units_updated.update(
+                                {c: {
+                                    'label': units[
+                                        unit]['*'].get('label'),
+                                    'decimals': units[
+                                        unit]['*'].get('decimals')
+                                }}
+                            )
+                        categories[idx][unit]['category'].update(
+                            {"unit": units_updated})
+                    else:
+                        categories[idx][unit]['category'].update(
+                            {"unit": units[unit]})
+
+    return categories
+
+
+def _get_categories(data, unit=None, role=None, value='value'):
+    """Return a list of categories according to dimensions.
+
+    When unit, role and value are included in parameters,
+    decimals are rounded, also adds units dictionary to result.
+
+    Args:
+      data(pandas.Dataframe): must have exactly one value column.
+      value (string, optional): name of the value column. Defaults to 'value'.
+      role(dict, optional): roles for dimensions.
+      unit(dict, optional): unit for variables, if there is only one
+                            element named '*' it will repeated for all.
+    """
+    dims = data.filter([item for item in data.columns.values
+                        if item not in value])
+    if isinstance(unit, dict) and isinstance(role, dict):
+        data = _round_decimals(data, unit, role, value)
+        categories_without_units = [
+            {to_int(i):
+                {"label": to_str(i),
+                 "category":
+                 {"index":
+                  OrderedDict([(to_str(j), to_int(k))
+                              for k, j in
+                              enumerate(uniquify(dims[i]))]),
+                  "label":
+                  OrderedDict([(to_str(j), to_str(j))
+                              for k, j in
+                              enumerate(uniquify(dims[i]))])
+                  }}}
+            for i in dims.columns.values]
+        categories = _add_units_to_categories(
+            categories_without_units, unit, role)
+    else:
+        categories = [{to_int(i):
+                       {"label": to_str(i),
+                        "category":
+                        {"index":
+                         OrderedDict([(to_str(j), to_int(k))
+                                      for k, j in
+                                      enumerate(uniquify(dims[i]))]),
+                         "label":
+                         OrderedDict([(to_str(j), to_str(j))
+                                      for k, j in
+                                      enumerate(uniquify(dims[i]))])}}}
+                      for i in dims.columns.values]
+
+    return categories
+
+
 def to_json_stat(input_df, value='value',
                  output='list', version='1.3',
-                 updated=datetime.today(), source='Self-elaboration'):
+                 updated=datetime.today(), source='Self-elaboration',
+                 note=None, role=None, unit=None):
     """Encode pandas.DataFrame object into JSON-stat format.
 
     The DataFrames must have exactly one value column.
@@ -453,7 +575,11 @@ def to_json_stat(input_df, value='value',
                        backwards compatibility.
       updated(datetime): updated metadata in JSON-stat standard. Must be a
                          datetime in ISO format.
-      source(string): data source in JSON-stat standard.
+      source(string, optional): data source in JSON-stat standard.
+      note(string, optional): information for metadata.
+      role(dict, optional): roles for dimensions.
+      unit(dict, optional): unit for variables, if there is only one
+                             element named '*' it will repeated for all.
 
     Returns:
       output(string): String with JSON-stat object.
@@ -479,18 +605,21 @@ def to_json_stat(input_df, value='value',
         if len(dims.columns.values) != len(set(dims.columns.values)):
             raise ValueError('Non-value columns must constitute a unique ID')
         dim_names = list(dims)
-        categories = [{to_int(i):
-                       {"label": to_str(i),
-                        "category":
-                        {"index":
-                         OrderedDict([(to_str(j), to_int(k))
-                                      for k, j in
-                                      enumerate(uniquify(dims[i]))]),
-                         "label":
-                         OrderedDict([(to_str(j), to_str(j))
-                                      for k, j in
-                                      enumerate(uniquify(dims[i]))])}}}
-                      for i in dims.columns.values]
+
+        if isinstance(role, dict):
+            for metric in role.get('metric'):
+                uniques_dimensions = dataframe[metric].unique()
+                for dimension in uniques_dimensions:
+                    tmp_df = dataframe.query(metric+" == '"+dimension+"'")
+                    time_column = tmp_df.loc[:, role.get('time')[0]]
+                    if time_column.duplicated().any():
+                        warnings.warn(
+                            "Data duplicated in time dimension."
+                            "Doesn't exist unique values for each dimension. ",
+                            UserWarning
+                        )
+
+        categories = _get_categories(dataframe, unit, role, value)
         if float(version) >= 2.0:
 
             dataset = {"dimension": OrderedDict(),
@@ -501,6 +630,10 @@ def to_json_stat(input_df, value='value',
             dataset["class"] = "dataset"
             dataset["updated"] = updated.isoformat()
             dataset["source"] = source
+            if isinstance(role, dict):
+                dataset["role"] = role
+            if isinstance(note, str):
+                dataset["note"] = [note]
             for category in categories:
                 dataset["dimension"].update(category)
             dataset.update({"id": dim_names})
@@ -577,7 +710,7 @@ class Dataset(OrderedDict):
         """Initialize object."""
         super(Dataset, self).__init__(*args, **kwargs)
 
-    @classmethod
+    @ classmethod
     def read(cls, data, verify=True, **kwargs):
         """Read data from URL, Dataframe, JSON string/file or OrderedDict.
 
@@ -742,7 +875,7 @@ class Dimension(OrderedDict):
         """Initialize object."""
         super(Dimension, self).__init__(*args, **kwargs)
 
-    @classmethod
+    @ classmethod
     def read(cls, data):
         """Read data from URL, Dataframe, JSON string/file or OrderedDict.
 
@@ -812,7 +945,7 @@ class Collection(OrderedDict):
         """Initialize object."""
         super(Collection, self).__init__(*args, **kwargs)
 
-    @classmethod
+    @ classmethod
     def read(cls, data):
         """Read data from URL or OrderedDict.
 
